searchData={"items":[{"type":"module","title":"Uplink","doc":"A simple abstraction for standardized observability with telemetry and more.\n\nUplink provides a simple abstraction for configuring observability for libraries\nand applications. The heart of Uplink is the `Uplink.Monitor` which provides a\nstandard template for common observability needs such as creating Telemetry.Metrics\ndefinitions, telemetry pollers, or setting up other custom observability requirements.\n\nThe most common challenge when getting started with telemetry is understanding where\nto start, what all the libraries do, and how they all fit together. This creates a\nhigh learning curve while leaving a huge gap at organizations where most developers\njust want a simple \"drop-in\" solution for observability that meets the org's requirements.\n\nUplink can be used on its own in simple personal projects or as the basis for a standard\n\"drop-in\" library with a simple abstraction containing standard monitors for your\norganization, such as your Telemetry.Metrics reporter of choice, BEAM VM measurements,\nor Phoenix measurements, all conforming to your org's metric naming conventions or\nother standard practices.","ref":"Uplink.html"},{"type":"module","title":"Usage - Uplink","doc":"```\n# application supervisor\nchildren = [\n  {\n    Uplink, [\n      monitors: [\n        {MyMonitors.Ecto, [:my_repo]},\n        Uplink.Monitors.VM\n      ],\n      pollers: [\n        {10, [{TestModule, :test_emitter, []}]}\n      ],\n      metric_definitions: [\n        Telemetry.Metrics.counter(\"poller.test.event.lasers\")\n      ],\n      reporters: [\n        Telemetry.Metrics.ConsoleReporter\n      ]\n    ]\n  }\n]\n```","ref":"Uplink.html#module-usage"},{"type":"function","title":"Uplink.child_spec/1","doc":"Returns the child spec for running Uplink under a supervisor.\n\nExample:\n\n    children = [\n      {Uplink, options}\n    ]\n\nSee `t:options/0` for a list of available options.","ref":"Uplink.html#child_spec/1"},{"type":"type","title":"Uplink.measurement/0","doc":"An MFA tuple for the poller to execute.","ref":"Uplink.html#t:measurement/0"},{"type":"type","title":"Uplink.metric_definitions/0","doc":"A list of `t:TelemetryMetrics.t/0` definitions.","ref":"Uplink.html#t:metric_definitions/0"},{"type":"type","title":"Uplink.monitor/0","doc":"A module or two-element tuple consisting of a module and arguments to be\nsupplied to the monitor. Any arguments passed are passed to all callbacks\nin the monitor.","ref":"Uplink.html#t:monitor/0"},{"type":"type","title":"Uplink.monitors/0","doc":"","ref":"Uplink.html#t:monitors/0"},{"type":"type","title":"Uplink.option/0","doc":"","ref":"Uplink.html#t:option/0"},{"type":"type","title":"Uplink.options/0","doc":"Valid options. No options are required but a monitor and/or monitor are the\nminimum required to do anything.\n\n* `:metric_definitions` - a list of additional `t:Telemetry.Metrics.t/0` definitions not\nexposed by monitors.\n* `:monitors` - a list of `t:monitor/0` to use.\n* `:poller_specs` - a list of additoinal `t:poller_spec/0` not exposed by monitors.\n* `:reporters` - a list of `Telemetry.Metrics` reporters, usually 0 or 1.","ref":"Uplink.html#t:options/0"},{"type":"type","title":"Uplink.period/0","doc":"Time in ms between poller executions.\n\nExample: `:timer.seconds(5)`","ref":"Uplink.html#t:period/0"},{"type":"type","title":"Uplink.poller_spec/0","doc":"A shorthand specification for listing a number of pollers to be executed at\na shared interval.\n\nExample: `{:timer.seconds(5), [{MyApp.Telemetry, :emit_stats, []}]}`","ref":"Uplink.html#t:poller_spec/0"},{"type":"type","title":"Uplink.poller_specs/0","doc":"","ref":"Uplink.html#t:poller_specs/0"},{"type":"type","title":"Uplink.reporter_spec/0","doc":"A module or two-element tuple consisting of a reporter and arguments to be\npassed to the reporter,\n\nExample: `{TelemetryMetricsPrometheus, [port: 9568]}`","ref":"Uplink.html#t:reporter_spec/0"},{"type":"type","title":"Uplink.reporter_specs/0","doc":"","ref":"Uplink.html#t:reporter_specs/0"},{"type":"behaviour","title":"Uplink.Monitor","doc":"A behaviour module for implementing a library or application monitor.\n\nUplink Monitors provide a template for the most common requirements to instrument a library\nor application in a consistent manner. These provide a simple abstraction for building\nstandard observability patterns for whatever is being monitored while removing the need\nfor users to repeatedly define and configure things like `TelemetryMetrics` definitions.\n\nThere are three required callbacks which make up a monitor. The provided macro provides\ndefault implementations for each, allowing you to only implement what is needed for a\nparticular monitor.\n\nEach callback receives the same argument defined in `t:Uplink.monitor/0`.","ref":"Uplink.Monitor.html"},{"type":"behaviour","title":"Usage - Uplink.Monitor","doc":"defmodule MyApp.CustomMonitor do\n      using Uplink.Monitor\n      # override any callbacks you need\n\n      @impl true\n      def init(_) do\n        # some setup\n        :ok\n      end\n\n      @impl true\n      def metric_definitions(_) do\n        [Telemetry.Metrics.counter(\"some.event\")]\n      end\n\n      @impl true\n      def poller_specs(_) do\n        [\n          {:timer.seconds(5), [\n            {MyApp.Telemetry, :emit_stats, []}\n          ]}\n        ]\n    end\n\nSee `Uplink.Monitors.VM` for an example implementation.","ref":"Uplink.Monitor.html#module-usage"},{"type":"callback","title":"Uplink.Monitor.init/1","doc":"Invoked when Uplink is starting.\n\nIt is useful for initializing any custom monitoring for the library or application\nbeing monitored. Examples include creating `telemetry` handlers to log slow `Ecto` queries\nwhich exceed a threshold or starting an OpenTelemetry bridge library.\n\nIf the function returns `:error`, Uplink will exit.\n\nThis callback is required.","ref":"Uplink.Monitor.html#c:init/1"},{"type":"callback","title":"Uplink.Monitor.metric_definitions/1","doc":"Invoked when Uplink is starting.\n\nIt is useful for providing a standard set of `t:Telemetry.Metrics.t/0` definitions\nfor the library or application being monitored.\n\nThis callback is required.","ref":"Uplink.Monitor.html#c:metric_definitions/1"},{"type":"callback","title":"Uplink.Monitor.poller_specs/1","doc":"Invoked when Uplink is starting.\n\nIt is useful for providing a standard set of `telemetry_poller`s for the library or\napplication being monitored. Examples include emitting cache sizes, memory usage, or\nprocess counts.\n\nThis callback is required.","ref":"Uplink.Monitor.html#c:poller_specs/1"},{"type":"module","title":"Uplink.Monitors.VM","doc":"Provides telemetry events, pollers for emitting those events, and definitions for\nBEAM VM statistics in addition to the builtins available `telemetry_poller` vm statistics.","ref":"Uplink.Monitors.VM.html"},{"type":"module","title":"Telemetry - Uplink.Monitors.VM","doc":"* `[:vm, :stats]`\n  * Description: Emits BEAM system statistics\n  * Measurements: `%{\n  context_switches: non_neg_integer,\n  gc_count: non_neg_integer,\n  gc_bytes_reclaimed: non_neg_integer,\n  io_in: non_neg_integer,\n  io_out: non_neg_integer,\n  reductions: non_neg_integer\n}\n`\n  * Metadata: `%{}`\n\n* `[:vm, :system_limits]`\n  * Description: Emits BEAM system limits\n  * Measurements: `%{\n  atom_limit: non_neg_integer,\n  port_limit: non_neg_integer,\n  process_limit: non_neg_integer\n}\n`\n  * Metadata: `%{}`","ref":"Uplink.Monitors.VM.html#module-telemetry"},{"type":"function","title":"Uplink.Monitors.VM.emit_stats/0","doc":"","ref":"Uplink.Monitors.VM.html#emit_stats/0"},{"type":"function","title":"Uplink.Monitors.VM.emit_system_limits/0","doc":"","ref":"Uplink.Monitors.VM.html#emit_system_limits/0"},{"type":"extras","title":"README","doc":"[![EEF Observability WG project](https://img.shields.io/badge/EEF-Observability-black)](https://github.com/erlef/eef-observability-wg)\n[![Hex.pm](https://img.shields.io/hexpm/v/uplink)](https://hex.pm/packages/uplink)\n![Build Status](https://github.com/elixir-uplink/uplink/workflows/Tests/badge.svg)\n\n# Uplink\n\nUplink makes setup of application monitoring with telemetry as simple as possible. Rather\nthan having to add and setup several libraries, this package rolls them all up into a single\nconfigurable drop-in to your application tree.","ref":"readme.html"},{"type":"extras","title":"Installation - README","doc":"```elixir\ndef deps do\n  [\n    {:uplink, \"~> 0.2\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - README","doc":"Add Uplink to your application supervision tree and tell it which monitors\nit should run. \n\n```elixir\n# application supervisor\nchildren = [\n  {\n    Uplink, [\n      monitors: [\n        {MyMonitors.Ecto, [repo_prefix: :my_repo]},\n        Uplink.Monitors.VM\n      ],\n      pollers: [\n        {10, [{TestModule, :test_emitter, []}]}\n      ],\n      metric_definitions: [\n        Telemetry.Metrics.counter(\"poller.test.event.lasers\")\n      ],\n      reporters: [\n        Telemetry.Metrics.ConsoleReporter\n      ]\n    ]\n  }\n]\n```\nSee the [docs](https://hexdocs.pm/uplink) for more information.","ref":"readme.html#usage"},{"type":"extras","title":"Uplink Monitors - README","doc":"Monitors by the community, for the community!","ref":"readme.html#uplink-monitors"},{"type":"extras","title":"Usage - README","doc":"Uplink Monitors are meant to be copied and pasted into your project as a starting point because \npreferences amongst individuals and organizations around metrics can vary widely. The important\nthing is to have a jumping-off point and can learn patterns and practices from others. These\naren't one-size-fits-all, so make them your own and share your learnings with others!\n\nIn the [monitors](https://github.com/elixir-uplink/monitors) folder you'll find monitors for popular\nlibraries which have been compiled over the past few years. They've worked quite well, so\nenjoy!\n\nCreate a PR to add a link to your own examples in the README or a monitor for a library\nnot already covered.\n\n#","ref":"readme.html#usage"},{"type":"extras","title":"Community Links - README","doc":"Copyright (c) 2021 Bryan Naegele","ref":"readme.html#community-links"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,49,46,50]}}